<?php

/**
 * Helper class.
 */
class TmgmtCdtHelper {

  /**
   * It's a json?
   */
  public static function isJson($data) {
        // @codingStandardsIgnoreStart
        if (is_string($data) && is_array(json_decode($data, TRUE)) && (json_last_error() === JSON_ERROR_NONE)) { 
            return TRUE;
        } else {
            return FALSE;
        }
        // @codingStandardsIgnoreEnd
  }

  /**
   * WatchdogDebug.
   */
  public static function watchdogDebug($var, $label = 'label') {

    $bt = debug_backtrace();
    $caller = array_shift($bt);

    watchdog(
        'watchdogDebug',
        '<pre>@file: @line</pre>@label: <pre>@var</pre>',
        array(
          '@label' => $label,
          '@var' => print_r($var, TRUE),
          '@file' => $caller['file'],
          '@line' => $caller['line'],
        ),
        WATCHDOG_DEBUG
    );
  }

  /**
   * All fields used in this module.
   *
   * @return array
   *   Fields definition.
   */
  public static function getFields() {
    return array(
      'tmgmt_job' => array(
        'cdt_correlation_id' => array(
          'label' => 'Correlation ID',
          'description' => 'ID returned after a successful request to CDT.',
          'db' => array(
            'type' => 'varchar',
            'length' => 32,
          ),
          'entity' => array(
            'type' => 'text',
          ),
        ),
        'cdt_request_identifier' => array(
          'label' => 'Request identifier',
          'description' => 'Seconde ID returned by CDT.',
          'db' => array(
            'type' => 'varchar',
            'length' => 32,
          ),
          'entity' => array(
            'type' => 'text',
          ),
        ),
        'cdt_request_params' => array(
          'label' => 'Request parameters',
          'description' => 'Parameters used to make the request to CDT.',
          'db' => array(
            'type' => 'text',
            'size' => 'medium',
          ),
          'entity' => array(
            'type' => 'text',
          ),
        ),
        'cdt_response' => array(
          'label' => 'Response',
          'description' => 'Response (translations) from CDT.',
          'db' => array(
            'type' => 'text',
            'size' => 'medium',
          ),
          'entity' => array(
            'type' => 'text',
          ),
        ),
      ),
      'tmgmt_job_item' => array(
        'cdt_source_language' => array(
          'label' => 'Source language',
          'description' => 'Source language.',
          'db' => array(
            'type' => 'varchar',
            'length' => 12,
          ),
          'entity' => array(
            'type' => 'text',
          ),
        ),
        'cdt_target_language' => array(
          'label' => 'Target language',
          'description' => 'Target language.',
          'db' => array(
            'type' => 'varchar',
            'length' => 12,
          ),
          'entity' => array(
            'type' => 'text',
          ),
        ),
        'cdt_character_count' => array(
          'label' => 'Character count',
          'description' => 'Character count.',
          'db' => array(
            'type' => 'int',
          ),
          'entity' => array(
            'type' => 'integer',
          ),
        ),
        'cdt_settings' => array(
          'label' => 'Settings',
          'description' => 'Settings.',
          'db' => array(
            'type' => 'text',
            'size' => 'medium',
            'serialize' => TRUE,
          ),
          'entity' => array(
            'type' => 'text',
          ),

        ),
      ),
    );
  }

  /**
   * Structure validation https://github.com/ptrofimov/matchmaker.
   *
   * @value array data to check
   * @pattern array rules
   */
  public static function validate($value, $pattern) {

    if (!function_exists('rules')) {

      /**
       * Rules
       */
      function rules($key = NULL) {
        static $rules;
        if (is_null($rules)) {
          $rules = [
                        /*
                         * General
                         */
            'empty' => 'empty',
            'nonempty' =>
            function ($value) {
              return !empty($value);
            },
                        'required' =>
                        function ($value) {
                          return !empty($value);
                        },
                        'in' =>
                        function ($value) {
                          return in_array($value, array_slice(func_get_args(), 1));
                        },
                        'mixed' =>
                        function () {
                          return TRUE;
                        },
                        'any' =>
                        function () {
                          return TRUE;
                        },
                        /*
                         * Types
                         */
                        'array' => 'is_array',
                        'bool' => 'is_bool',
                        'boolean' => 'is_bool',
                        'callable' => 'is_callable',
                        'double' => 'is_double',
                        'float' => 'is_float',
                        'int' => 'is_int',
                        'integer' => 'is_integer',
                        'long' => 'is_long',
                        'numeric' => 'is_numeric',
                        'number' => 'is_numeric',
                        'object' => 'is_object',
                        'real' => 'is_real',
                        'resource' => 'is_resource',
            'scalar' => 'is_scalar',
            'string' => 'is_string',
                        /*
                         * Numbers
                         */
            'gt' =>
            function ($value, $n) {
              return $value > $n;
            },
                        'gte' =>
                        function ($value, $n) {
                          return $value >= $n;
                        },
                        'lt' =>
                        function ($value, $n) {
                          return $value < $n;
                        },
                        'lte' =>
                        function ($value, $n) {
                          return $value <= $n;
                        },
                        'negative' =>
                        function ($value) {
                          return $value < 0;
                        },
                        'positive' =>
                        function ($value) {
                          return $value > 0;
                        },
                        'between' =>
                        function ($value, $a, $b) {
                          return $value >= $a && $value <= $b;
                        },
                        /*
                         * Strings
                         */
                        'alnum' => 'ctype_​alnum',
                        'alpha' => 'ctype_​alpha',
                        'cntrl' => 'ctype_​cntrl',
                        'digit' => 'ctype_​digit',
                        'graph' => 'ctype_​graph',
                        'lower' => 'ctype_​lower',
                        'print' => 'ctype_​print',
                        'punct' => 'ctype_​punct',
                        'space' => 'ctype_​space',
                        'upper' => 'ctype_​upper',
            'xdigit' => 'ctype_​xdigit',
            'regexp' =>
            function ($value, $regexp) {
              return preg_match($regexp, $value);
            },
                        'email' =>
                        function ($value) {
                          return filter_var($value, FILTER_VALIDATE_EMAIL);
                        },
                        'url' =>
                        function ($value) {
                          return filter_var($value, FILTER_VALIDATE_URL);
                        },
                        'ip' =>
                        function ($value) {
                          return filter_var($value, FILTER_VALIDATE_IP);
                        },
                        'length' =>
                        function ($value, $length) {
                          return mb_strlen($value, 'utf-8') === $length;
                        },
                        'min' =>
                        function ($value, $min) {
                          return mb_strlen($value, 'utf-8') >= $min;
                        },
                        'max' =>
                        function ($value, $max) {
                          return mb_strlen($value, 'utf-8') <= $max;
                        },
                        'contains' =>
                        function ($value, $needle) {
                          return strpos($value, $needle) !== FALSE;
                        },
                        'starts' =>
                        function ($value, $string) {
                          return mb_substr($value, 0, mb_strlen($string, 'utf-8'), 'utf-8') === $string;
                        },
                        'ends' =>
                        function ($value, $string) {
                          return mb_substr($value, -mb_strlen($string, 'utf-8'), 'utf-8') === $string;
                        },
                        'json' =>
                        function ($value) {
                            // @codingStandardsIgnoreStart
                            return @json_decode($value) !== NULL;
                            // @codingStandardsIgnoreEnd
                        },
                        'date' =>
                        function ($value) {
                          return strtotime($value) !== FALSE;
                        },
                        /*
                         * Arrays
                         */
                        'count' =>
                        function ($value, $count) {
                          return is_array($value) && count($value) === $count;
                        },
                        'keys' =>
                        function ($value) {
                          if (!is_array($value)) {
                            return FALSE;
                          }
                          foreach (array_slice(func_get_args(), 1) as $key) {
                            if (!array_key_exists($key, $value)) {
                              return FALSE;
                            }
                          }
                          return TRUE;
                        },
                        /*
                         * Objects
                         */
                        'instance' =>
                        function ($value, $class) {
                          return is_object($value) && $value instanceof $class;
                        },
                        'property' =>
                        function ($value, $property, $expected) {
                          return
                            is_object($value)
                            && (property_exists($value, $property) || property_exists($value, '__get'))
                            && $value->$property === $expected;
                        },
                        'method' =>
                        function ($value, $method, $expected) {
                          return
                            is_object($value)
                            && (method_exists($value, $method) || method_exists($value, '__call'))
                            && $value->$method() === $expected;
                        },
    ];
        }
        if (is_array($key)) {
          $rules = array_merge($rules, $key);
        }
        elseif (!is_null($key)) {
          if (is_string($key) && !trim($key)) {
            $key = 'any';
          }

          if (!isset($rules[$key])) {
            throw new \InvalidArgumentException("Matcher $key not found");
          }
          return $rules[$key];
        }
        return $rules;
      }

    }

    if (!function_exists('matcher')) {

      /**
       * matcher
       */
      function matcher($value, $pattern) {

        $args = [];
        if (($p = ltrim($pattern, ':')) != $pattern) {
          foreach (explode(' ', $p) as $name) {
            if (drupal_substr($name, -1) === ')') {
              list($name, $args) = explode('(', $name);
              $args = explode(',', rtrim($args, ')'));
            }
            if (is_callable(rules($name))) {
              if (!call_user_func_array(rules($name), array_merge([$value], $args))) {
                echo "FALSE $name $value <br>";
                return FALSE;
              }
            }
            elseif (rules($name) !== $value) {
              echo "FALSE $name $value <br>";
              return FALSE;
            }
          }
        }
        else {

          $result = $pattern === '' || $value === $pattern;
          if ($result) {
            echo "TRUE $pattern $value <br>";
          }
          else {
            echo "FALSE $pattern $value <br>";
          }

          return $result;
        }
        echo "TRUE $pattern $value <br>";
        return TRUE;
      }

    }

    if (!function_exists('key_matcher')) {

      /**
       * key_matcher
       */
      // @codingStandardsIgnoreStart
      function key_matcher(array $pattern) {
        // @codingStandardsIgnoreEnd
        $keys = [];
        foreach ($pattern as $k => $v) {
          $chars = ['?' => [0, 1], '*' => [0, PHP_INT_MAX], '!' => [1, 1]];
          if (isset($chars[$last = drupal_substr($k, -1)])) {
            $keys[$k = drupal_substr($k, 0, -1)] = $chars[$last];
          }
          elseif ($last === '}') {
            list($k, $range) = explode('{', $k);
            $range = explode(',', rtrim($range, '}'));
            $keys[$k] = count($range) === 1
                        ? [$range[0], $range[0]]
                        : [$range[0] === '' ? 0 : $range[0], $range[1] === '' ? PHP_INT_MAX : $range[1]];
          }
          else {
            $keys[$k] = $chars[$k[0] === ':' ? '*' : '!'];
          }
          array_push($keys[$k], $v, 0);
        }
        return function ($key = NULL, $value = NULL) use (&$keys) {
          if (is_null($key)) {
            foreach ($keys as $count) {
              if ($count[3] < $count[0] || $count[3] > $count[1]) {
                return FALSE;
              }
            }
          }
          else {
            foreach ($keys as $k => &$count) {
              if (matcher($key, $k)) {
                if (!TmgmtCdtHelper::validate($value, $count[2])) {
                  return FALSE;
                }
                $count[3]++;
              }
            }
          }

          return TRUE;
        };
      }

    }

    if (is_array($pattern)) {
      if (!is_array($value) && !$value instanceof \Traversable) {
        return FALSE;
      }
      $keyMatcher = key_matcher($pattern);
      foreach ($value as $key => $item) {
        if (!$keyMatcher($key, $item)) {
          return FALSE;
        }
      }
      if (!$keyMatcher()) {
        return FALSE;
      }
    }
    elseif (!matcher($value, $pattern)) {
      return FALSE;
    }
    return TRUE;
  }

}
